---
draft: true
pin: true
date: 2024-10-24
authors:
  - chensy
categories:
  - 碎碎念
---


多因子+机器学习训练OK后生成模型给智能体调用，智能根据信号直接调用QMT交易


多因子+机器学习模型设计：

一、数据准备与预处理

1. 数据获取：
    使用`tushare`等金融数据接口获取股票的历史交易数据，包括开盘价、收盘价、最高价、最低价、成交量、成交额等基础数据。同时，获取财务报表数据、行业分类信息、宏观经济数据等相关数据。
    收集新闻资讯、社交媒体数据、分析师报告等非结构化数据，通过自然语言处理技术提取有价值的信息，如题材热度、市场情绪等。

2. 数据清洗：
    处理缺失值：对于连续型变量，可以使用均值、中位数或线性插值等方法进行填充；对于离散型变量，可以使用众数或特殊值进行填充。
    处理异常值：通过箱线图、3σ原则等方法识别异常值，并进行适当的处理，如删除、替换或 Winsorize 处理。
    数据标准化：对不同量纲的数据进行标准化处理，如 Zscore 标准化、MinMax 标准化等，使各因子具有可比性。

3. 特征工程：
    构建多因子体系：根据前面提到的资金流、盘口订单簿、缠论、题材、热点和情绪等维度，构建多个因子。例如：
      资金流因子：计算资金净流入、资金流入流出强度、大单交易占比等。
      盘口订单簿因子：分析盘口深度、价格分布、价差、大单动向等。
      缠论因子：识别走势类型、中枢区间、背驰情况等。
      题材因子：计算题材热度、题材分类与投资者风格适配度等。
      热点因子：确定热点强度、热点识别等。
      情绪因子：衡量市场情绪、投资者情绪等。
    时间序列特征提取：对于历史交易数据，可以提取移动平均线、相对强弱指标（RSI）、布林带等技术指标作为特征。
    文本特征提取：对于非结构化数据，使用自然语言处理技术提取关键词、情感倾向、主题等特征。

二、单因子分析与筛选

1. 单因子评估指标：
    信息系数（IC）：计算因子值与股票未来收益之间的相关性，IC 值越高，说明因子的预测能力越强。
    分层回测：根据因子值将股票分成若干组，计算每组的平均收益和累计收益，观察不同组之间的收益差异。如果因子具有较好的区分度，那么高分组和低分组的收益差异应该较大。
    多空组合收益：构建多空组合，即买入因子值最高的一组股票，卖出因子值最低的一组股票，计算多空组合的收益。多空组合收益越高，说明因子的有效性越强。

2. 单因子筛选：
    基于评估指标进行筛选：选择 IC 值较高、分层回测效果好、多空组合收益高的因子。
    相关性分析：计算各因子之间的相关性，避免选择相关性过高的因子，以减少冗余信息。
    稳定性检验：观察因子在不同时间段的表现，选择稳定性较好的因子。

三、多因子模型构建

1. 因子加权方法：
    等权加权：对筛选后的因子赋予相同的权重，简单易行，但可能无法充分发挥各因子的优势。
    最优化加权：使用线性回归、主成分分析（PCA）、因子分析等方法确定各因子的最优权重，以最大化多因子模型的预测能力。

2. 模型训练与验证：
    划分数据集：将历史数据划分为训练集、验证集和测试集。训练集用于模型训练，验证集用于调整模型参数和选择最优模型，测试集用于评估模型的最终性能。
    模型选择：可以选择线性回归、逻辑回归、支持向量机（SVM）、随机森林、深度学习等机器学习算法构建多因子模型。
    超参数调优：使用网格搜索、随机搜索、贝叶斯优化等方法对模型的超参数进行调优，以提高模型的性能。
    模型评估：使用多种评估指标对多因子模型进行评估，如准确率、召回率、F1 值、均方误差（MSE）、夏普比率等。

四、机器学习模型优化

1. 特征选择：
    过滤式特征选择：使用方差分析（ANOVA）、互信息（MI）等方法对特征进行筛选，去除无关或冗余特征。
    包裹式特征选择：使用递归特征消除（RFE）、遗传算法等方法，通过不断地添加或删除特征来寻找最优的特征子集。
    嵌入式特征选择：在模型训练过程中自动进行特征选择，如 Lasso 回归、岭回归等。

2. 模型融合：
    简单平均法：对多个不同的机器学习模型进行简单平均，得到最终的预测结果。
    加权平均法：根据各模型的性能表现，为不同的模型赋予不同的权重，进行加权平均。
    堆叠法（Stacking）：将多个不同的模型作为基学习器，再使用一个元学习器对基学习器的预测结果进行整合。

3. 深度学习模型应用：
    构建深度神经网络（DNN）、卷积神经网络（CNN）、循环神经网络（RNN）等深度学习模型，利用其强大的特征提取和学习能力，对股票数据进行建模。


# -*- coding: utf-8 -*-
# @Time    : 2023/7/7  22:48
# @FileName: Config.py
# @Author  : Loyeller
"""
描述: 参数配置文件
"""


# -------- 策略名 --------
strategy_name = '小市值_过滤优化'

# -------- 复权配置 --------
fuquan_type = '后复权'

# -------- 选股参数设定 --------
period_type = 'W'  # W代表周，M代表月
date_start = '2010-01-01'  # 数据开始时间，一般要比回测开始时间久
date_end = '2023-04-01'

select_stock_num = 20           # 选股数量
# select_stock_num = 5 / 100.0    # 百分比选股

c_rate = 1.2 / 10000    # 手续费
t_rate = 1 / 1000       # 印花税

# -------- 数据存放路径 --------
# 股票日线数据
stock_data_path = r'E:\data\stock_xbx\stock_kdata_daily'
# stock_data_path = r'F:\data2\stock_xbx\stock_kdata_daily'

# 财务数据路径
finance_data_path = r'E:\data\stock_xbx\stock_finance'
# finance_data_path = r'F:\data2\stock_xbx\stock_finance'

# 指数数据路径
index_path = r'E:\data\stock_xbx\index_kdata\sz399317.csv'
# index_path = r'F:\data2\stock_xbx\index_kdata\sh000300.csv'

# 整理好的选股数据存放路径
selection_data_path = r'E:\data\stock_xbx\stock_selection'
# selection_data_path = r'F:\data2\stock_xbx\stock_selection'

# -------- 财务指标准备(只准备指定的指标) --------
# 因为财务数据众多，将本策略中需要用到的财务数据字段罗列如下
raw_fin_cols = [
    'R_operating_total_revenue@xbx',    # 营业总收入_流量型
    'R_operating_total_cost@xbx',       # 营业总成本_流量型
    'R_revenue@xbx',                    # 营业收入_流量型
    'R_operating_cost@xbx',             # 营业成本_流量型
    'R_total_profit@xbx',               # 利润总额_流量型
    'R_np@xbx',                         # 净利润_流量型
    'R_np_atoopc@xbx',                  # 归母净利润_流量型
    'R_non_operating_income@xbx',       # 营业外收入_流量型
    'R_nonoperating_cost@xbx',          # 营业外支出_流量型
    'R_income_tax_cost@xbx',            # 所得税_流量型
    'C_ncf_from_oa@xbx',                # 经营活动净现金流_流量型
    'C_net_increase_in_cce@xbx',        # 净现金流_流量型

    'B_total_owner_equity@xbx',         # 归母所有者权益合计_截面型
    'B_total_current_liab@xbx',         # 流动负债_截面型
    'B_total_current_assets@xbx',       # 流动资产_截面型
    'B_inventory@xbx',                  # 流动资产的存货_截面型
]




# 指定流量字段flow_fin_cols和截面字段cross_fin_cols。flow_fin_cols、cross_fin_cols必须是raw_fin_cols的子集

# 指定流量型指标
flow_fin_cols = [
    'R_operating_total_revenue@xbx',    # 营业总收入_流量型
    'R_operating_total_cost@xbx',       # 营业总成本_流量型
    'R_revenue@xbx',                    # 营业收入_流量型
    'R_operating_cost@xbx',             # 营业成本_流量型
    'R_total_profit@xbx',               # 利润总额_流量型
    'R_np@xbx',                         # 净利润_流量型
    'R_np_atoopc@xbx',                  # 归母净利润_流量型
    'R_non_operating_income@xbx',       # 营业外收入_流量型
    'R_nonoperating_cost@xbx',          # 营业外支出_流量型
    'R_income_tax_cost@xbx',            # 所得税_流量型
    'C_ncf_from_oa@xbx',                # 经营活动净现金流_流量型
    'C_net_increase_in_cce@xbx',        # 净现金流_流量型
]

# 指定截面型
cross_fin_cols = [
    'B_total_owner_equity@xbx',         # 所有者权益合计_截面型
    'B_total_current_liab@xbx',         # 流动负债_截面型
    'B_total_current_assets@xbx',       # 流动资产_截面型
    'B_inventory@xbx',                  # 流动资产的存货_截面型
]

# 财务指标的衍生指标
# 流量型指标支持: _单季, _ttm, _单季环比, _单季同比, _累计同比, _ttm同比
# 截面型指标支持: _环比, _同比
derived_fin_cols = [
    # 单季指标
    'R_operating_total_revenue@xbx_单季',     # 营业总收入_流量型_单季
    'R_operating_total_cost@xbx_单季',        # 营业总成本_流量型_单季
    'R_revenue@xbx_单季',                     # 营业收入_流量型_单季
    'R_operating_cost@xbx_单季',              # 营业成本_流量型_单季
    'R_non_operating_income@xbx_单季',        # 营业外收入_流量型_单季
    'R_nonoperating_cost@xbx_单季',           # 营业外支出_流量型_单季
    'R_total_profit@xbx_单季',                # 利润总额_流量型_单季
    'R_np@xbx_单季',                          # 净利润_流量型_单季
    'R_np_atoopc@xbx_单季',                   # 归母净利润_流量型_单季
    'C_ncf_from_oa@xbx_单季',                 # 经营活动净现金流_流量型_单季
    'C_net_increase_in_cce@xbx_单季',         # 净现金流_流量型_单季
    'R_income_tax_cost@xbx_单季',             # 所得税_流量型_单季

    
    # ttm类指标
    'R_operating_total_revenue@xbx_ttm',    # 营业总收入_流量型_ttm
    'R_operating_total_cost@xbx_ttm',       # 营业总成本_流量型_ttm
    'R_revenue@xbx_ttm',                    # 营业收入_流量型_ttm
    'R_operating_cost@xbx_ttm',             # 营业成本_流量型_ttm
    'R_non_operating_income@xbx_ttm',       # 营业外收入_流量型_ttm
    'R_nonoperating_cost@xbx_ttm',          # 营业外支出_流量型_ttm
    'R_total_profit@xbx_ttm',               # 利润总额_流量型_ttm
    'R_np@xbx_ttm',                         # 净利润_流量型_ttm
    'R_np_atoopc@xbx_ttm',                  # 归母净利润_流量型_ttm
    'C_ncf_from_oa@xbx_ttm',                # 经营活动净现金流_流量型_ttm
    'C_net_increase_in_cce@xbx_ttm',        # 净现金流_流量型_ttm
    'R_income_tax_cost@xbx_ttm',            # 所得税_流量型_ttm

    # 成长类因子
    'R_revenue@xbx_单季同比',                 # 营业收入_流量型_单季同比
    'R_revenue@xbx_ttm同比',                 # 营业收入_流量型_ttm同比
    'R_np_atoopc@xbx_单季同比',               # 归母净利润_流量型_单季同比
    'R_np_atoopc@xbx_ttm同比',               # 归母净利润_流量型_ttm同比
    'C_ncf_from_oa@xbx_单季同比',             # 经营活动净现金流_流量型_单季同比
    'C_ncf_from_oa@xbx_ttm同比',             # 经营活动净现金流_流量型_ttm同比
]

